// vim: foldmethod=marker

//var gShelfCommandController = null;

flock.shelf = {};

flock.shelf.__defineGetter__("service", function () {/*{{{*/
    if (flock.shelf._service) return flock.shelf._service;
    flock.shelf._service = Components.classes['@mozilla.org/rdf/datasource;1?name=flock-shelf']
        .getService(Components.interfaces['flockIScrapbookService']);
    return flock.shelf._service;
});/*}}}*/

flock.shelf.__defineGetter__("datasource", function () {/*{{{*/
    if(flock.shelf._datasource) return flock.shelf._datasource;
    flock.shelf._datasource = new RDFDataSource();
    flock.shelf._datasource.Init(flock.shelf.service);
    return flock.shelf._datasource;
});/*}}}*/


// RichObject class /*{{{*/
function RichObject() {}

RichObject.prototype = {
    init: function(type, title, content, webPageTitle, webPageUrl, objectUrl){
        this.type = type;
        this.title = title;
        this.content = content;
        this.url = objectUrl;
        this.webPageTitle = webPageTitle;
        this.webPageUrl = webPageUrl;
    },
    getInterfaces: function (count) {
        var interfaceList = [flockIRichObject, nsIClassInfo];
        count.value = interfaceList.length;
        return interfaceList;
    },
    toHtml: function() {
        return "";
    },
    toText: function() {
        return "";
    }
}
/*}}}*/

// RichSelection class /*{{{*/
var RichSelection = function() {
   this.init = function(selection, webPageUrl, webPageTitle){
        this.type = "document";
        this.title = selection.toString().replace(/\n/g, ' ');
        this.content = selection.toString();
        this.url = webPageUrl;
        this.webPageTitle = webPageTitle;
        this.webPageUrl = webPageUrl;
   };
   this.toHtml = function() {
        return '<blockquote cite="' + this.webPageTitle + '">'
               + this.content + "</blockquote>"
               + '<p class="citation"><cite cite="' + this.webPageUrl + '">'
               + '<a href="' + this.webPageUrl + '">' + this.webPageTitle + '</a>'
               + "</cite></p><br/>";
    };
   this.toText = function() {
       return this.selection + "\n\n" + this.webPageTitle + "(" + this.webPageUrl + ")";
   }
}

RichSelection.prototype = new RichObject();
/*}}}*/

// RichImage class /*{{{*/
var RichImage = function() {
   this.init = function(title, imageUrl, webPageUrl){
        this.type = "image";
        this.title = title;
        this.content = "";
        this.url = imageUrl;
        this.webPageTitle = "";
        this.webPageUrl = webPageUrl;
   };
   this.toHtml = function() {
        return '<a href="' + this.webPageUrl + '" title="' + this.title + '"><img border="0" alt="'
               + this.title + '" src="' + this.url + '"/></a>';
   };
   this.toText = function() {
       return title + ": " + imageUrl;
   }
}

RichImage.prototype = new RichObject();
/*}}}*/

// RichLink class /*{{{*/
var RichLink = function() {
   this.init = function(content, webPageUrl){
        this.type = "link";
        this.title = content.replace(/\n/g, ' ');
        this.content = content;
        this.url = webPageUrl;
        this.webPageTitle = title;
        this.webPageUrl = webPageUrl;
   };
   this.toHtml = function() {
        return '<a href="' + this.webPageUrl + '"/>'+this.content+'</a>';
   };
   this.toText = function() {
       return content + ": " + this.webPageUrl;
   }
}

RichLink.prototype = new RichObject();
/*}}}*/

// RichNote class /*{{{*/
var RichNote = function() {
   this.init = function(content){
        this.type = "note";
        this.content = content;
        this.title = content.replace(/\n/g, ' ');
        this.url = "";
        this.webPageTitle = "";
        this.webPageUrl = "";
   };
   this.toHtml = function() {
        return this.content;
   };
   this.toText = function() {
       return this.content;
   }
}

RichNote.prototype = new RichObject();
/*}}}*/


flock.shelf.controller = function (aNode, aPanel, aPanelCreate, aPanelClear, aDeck, aView) {/*{{{*/
    this.mNode = aNode;
    this.mPanel = aPanel;
    this.mPanelCreate = aPanelCreate;
    this.mPanelClear = aPanelClear;
    this.mDeck = aDeck;
    this.mView = aView;
    this.closeTimeout = null;
    this._previewMode = false;
    this._open = false;
};

flock.shelf.controller.prototype = {
    get previewMode() { /*{{{*/
        return this._previewMode; 
    },/*}}}*/
    set previewMode(aVal) { /*{{{*/
        this._previewMode = aVal;
        if (this.mPanel) this.mPanel.previewmode = aVal;
    },/*}}}*/
    open: function () {/*{{{*/
        this.mNode.hidden = false;
        if (this.mPanel) {
            this.mPanel.setAttribute("open", "true");
        }
        if (this.mPanelCreate) {
            this.mPanelCreate.setAttribute("hidden", "false");
        }
        if (this.mPanelClear) {
            this.mPanelClear.setAttribute("hidden", "false");
        }
        this._open = true;
    },/*}}}*/
    close: function () {/*{{{*/
        if (this.previewMode) {
            this.previewMode = false;
        }
        if (this.closeTimeout) {
            this.closeTimeout.cancel();
        }
        this.mNode.hidden = true;
        if (this.mPanel) {
            this.mPanel.removeAttribute("open");
        }
        if (this.mPanelCreate) {
            this.mPanelCreate.setAttribute("hidden", "true");
        }
        if (this.mPanelClear) {
            this.mPanelClear.setAttribute("hidden", "true");
        }
        this.mView.clearSelection();
        this._open = false;
    },/*}}}*/
    toggle: function () {/*{{{*/
        if (this._open) {
            this.close();
        }
        else {
            this.open();
        }
    },/*}}}*/
    preview: function () {/*{{{*/
        if (this.previewMode) {
            // bump the timer and return
            this.setCloseTimeout();
            return;
        }
        else if (this._open) {
            // if it was already open and we aren not
            // previewing, just leave it open
            return;
        }
        this.previewMode = true;
        this.open();
        this.setCloseTimeout();
    },/*}}}*/
    setCloseTimeout: function () {/*{{{*/
        if (this.closeTimeout) {
            this.closeTimeout.bump();
            return;
        }
        var _this = this;
        this.closeTimeout = new flock.xpcom.Timer(function (aEvent) {
            _this.close();
        }, 2000);
    },/*}}}*/
    // move to "No items" and "Items : X" for localization
    refreshPanel: function () {/*{{{*/
        if (!this.mPanel) return;
        var count = flock.shelf.service.elementsCount();
        switch (count) {
            case 0: message = "No Items"; 
                    this.mDeck.setAttribute('selectedIndex', '0'); break;
            case 1: message = "1 Item";
                    this.mDeck.setAttribute('selectedIndex', '1'); break;
            default: message = count + " Items";
                     this.mDeck.setAttribute('selectedIndex', '1');
        }
        //this.mPanel.setAttribute('tooltiptext', message);
    },/*}}}*/

    // util
    getSelection: function () {/*{{{*/
        return this.mView.getSelection();
    },/*}}}*/
    selectAll: function () {/*{{{*/
        this.mView.selectAll();
    },/*}}}*/
    clearSelection: function () {/*{{{*/
        this.mView.clearSelection();
    },/*}}}*/
    scrollHome: function () {/*{{{*/
        this.mView.scrollHome();
    },/*}}}*/
    scrollEnd: function () {/*{{{*/
        this.mView.scrollEnd();
    },/*}}}*/
    get canScrollRight() { return this.mView.canscrollright; },
    get canScrollLeft() { return this.mView.canscrollleft; },

    // contents
    addObject: function (aData) {/*{{{*/
        var data = aData;
        var title;
        var url;
        var pos;

        var theObject;
        switch(data.type){
            case 'document': 
                theObject = new RichSelection();
                theObject.init(data.label, data.header, data.webPageTitle);
                break;
            case 'image': 
                theObject = new RichImage();
                // Don't insert a stupid image containing Javascript
                if (data.label.match('javascript:'))
                  return;
                theObject.init(data.header, data.label, data.webPageUrl);
                break;
            case 'link': 
                theObject = new RichLink();
                theObject.init(data.header, data.label);
                break;
            case 'note': 
                theObject = new RichNote();
                theObject.init(data.header);
                break;
        }

        var hoverChild = this.mView.hoverChild;
        // for(var i in hoverChild){ XXX("hoverChild "+i, hoverChild[i]);}

        if (hoverChild){
            pos = flock.shelf.service.getPositionOf(hoverChild.firstChild.id);
        } else {
            pos = -1;
        }

        var itemId = flock.shelf.service.insert(theObject, pos);
        
        if (data.dragfrom) {
            flock.shelf.service.remove(data.dragfrom);
        }
        
        // we want to ensure that what was just added is visible
        var newPos = flock.shelf.service.getPositionOf(itemId);
        this.mView.ensureElementIsVisible(this.mView.childNodes[newPos]);
        
        // we also want to clear the selection
        this.clearSelection();

    },/*}}}*/

    
    // drag stuff
    onDragOverIcons: function (aEvt) {/*{{{*/
        this.preview();
    },/*}}}*/
    onDragOver: function (aEvt, aFlavour, aSession) {/*{{{*/
        // Files are not supported, and should not activate preview
        // ...but as a side-effect the "preview" mode is also disabled
        // for text coming from external applications and text forms
        if (aFlavour.contentType == "text/unicode")
            return;
        this.preview();
    },/*}}}*/
    getSupportedFlavours: function(){/*{{{*/
        flavors=new FlavourSet();
        flavors.appendFlavour('text/html');
        flavors.appendFlavour('text/unicode');
        flavors.appendFlavour('text/x-moz-url');
        return flavors;
    },/*}}}*/

    // XXX andy: this is a stupid hack to get around slideybox
    //           trying to handle its own keystrokes
    preventSlideyboxKeystrokesHack: function (aEvt) {/*{{{*/
        if (aEvt.keyCode == aEvt.DOM_VK_DELETE ||
            aEvt.keyCode == aEvt.DOM_VK_BACK_SPACE ||
            aEvt.keyCode == aEvt.DOM_VK_CLEAR ||
            aEvt.keyCode == aEvt.DOM_VK_A) {
            aEvt.stopPropagation();
        }
    },/*}}}*/
};/*}}}*/


function flush_shelf(){/*{{{*/
    if (flock.shelf.service.elementsCount() > 0){
        var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
            .getService(Components.interfaces.nsIPromptService);
        var strbundle = document.getElementById("shelfStringBundle");
        var msg = strbundle.getString("flock.shelf.menu.delete");
        if (prompts.confirm(window, strbundle.getString("flock.shelf.menu.delete.title"), msg)){
            flock.shelf.service.clear();
            $T('shelf-deck').setAttribute('selectedIndex','0');
        }
    }
}/*}}}*/

function add_note() {/*{{{*/
    window.openDialog("chrome://browser/content/flock/shelf/addShelfNoteDlg.xul", "addNote", "modal");
    $('icon-view').builder.refresh();
}/*}}}*/

// FIXME: clean this big fat function
function flock_getMarkedUpSelection(aEvt, aDropData, aSession){/*{{{*/
  var windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService();
  var windowManagerInterface = windowManager.QueryInterface( Components.interfaces.nsIWindowMediator);
  var topWindow = windowManagerInterface.getMostRecentWindow("navigator:browser");

  var data="";
  var node=aSession.sourceNode;
  var serializer = new XMLSerializer();
  var parser = new DOMParser();

  var header="Header";
  var label="Label";
  var type="document";

  var webPageUrl = topWindow.document.getElementById('content').contentDocument.URL;
  var webPageTitle = topWindow.document.getElementById('content').contentTitle;
  // Text (from input zone or external application)
  if (aDropData.flavour.contentType == "text/unicode") {
   var location = topWindow.document.getElementById('content').mCurrentBrowser.contentWindow.location + "";
   var text = aDropData.data;
   return { type: "note", header: text };
  }

  var contentDetector = new nsContentDetector();
  contentDetector.setTarget(node);

  // Images
  if (contentDetector.onImage) {
    header=node.alt;
    label=contentDetector.imageURL;
    if (label == webPageUrl) // Bug 3756, when the image alone is displayed in the browser
      header = label.split('/').pop();
    type='image';
    var src='<img src="' + label + '" />';
    return { type: type, header: header, label: label, src: src, webPageUrl: webPageUrl};
  }
    
	//Not handling files just yet.
	if(aDropData.flavour.contentType=='application/x-moz-file'){
		return null;
	}

	//For dragging from outside the browser. Not supported for now.
	if(!node){
		return null;
	}

  // XXX('markedselection', node.nodeName.toLowerCase());
  switch(node.nodeName.toLowerCase()){
	//Handle image maps
    case 'area':
		var map='#' + node.parentNode.getAttribute('name');
		var images=topWindow.window._content.document.getElementsByTagName('img');
		dump(images.length + '\n');
		for(i=0; i<images.length; i++){
			dump('IMG: ' + images[i].src + '\n');
			if(images[i].getAttribute('usemap')==map){
				node=images[i];
			}
		}
		if(node.nodeName.toLowerCase()=='area'){
			dump('Could not find image corresponding to map ' + map + '.\n');
			return false;
		}
		//TODO: Normalize paths for node...
		return { type: type, header: header, label: label, src: serializer.serializeToString(node) };
	
	// Big hack. This is for when somebody drags the favicon onto the shelf. 
    // Without this, we get the xul source b/c we are using sourceNode.
	case 'image':
		header=topWindow.document.getElementById('content').contentTitle;
		label=topWindow.document.getElementById('content').contentDocument.URL;
		type='link';
		var src='<a href="' + label + '">' + $$('content').contentTitle + '</a>';
		return { type: type, header: header, label: label, src: src };

	//Handle flickr photos
	case 'photo':
    header = node.title;
		header=header.replace(/%20/g, ' ');
    label = node.src;
    webPageUrl = node.webPageUrl;
		return { type: "image", header: header, label: label, src: node.photoHTML, webPageUrl: webPageUrl};
    // Browser tabs
    case 'tab':
    case 'xul:tab':
        return { type: "link", header: node.label, label: window.top.content.document.location.toString(), src: "" };
    // Can be from the shelf itself (to ignore)
    // or from the favorite manager (to insert)
    // XXX need to find a cleaner way to do all that stuff
    case 'shelficon':
         if (node.type == 'document'){
            return { type: node.type, header: node.url, label: node.title, src: node.src, dragfrom: node.id };
         } else {
            return { type: node.type, header: node.title, label: node.url, src: node.src, dragfrom: node.id };
         }
    case 'treechildren': // ERW: Big hack to get the info I need. Will be better once we have metadata attached to each object we drag
        XXX('node.parentNode.id', node.parentNode.id);
        if (node.parentNode.id == "flock_favoriteView")
        {
            var source = aDropData.data;
            var title =  source.split(">")[1].split("<")[0];
            var url =  source.split("'")[1];
            return { type: "link", header: title, label: url, src: source };
        } else {
        //    for (i in node.parentNode){
        //        XXX('treechildren.parentnode->'+i, node.parentNode[i]);
        //    }
        }
    default:
        //Handle selections	
        var selection = flock.browser.getSelection();
        for (i in node){
            XXX("node."+i, node[i]);
        }

        if(selection.toString().length > 0){
            //XXX('We get the selection ', selection);
            var selection_formatted=select_src(selection);
		    title=topWindow.document.getElementById('content').contentTitle;
		    if(title.length==0) title = location;

		    var location = topWindow.document.getElementById('content').mCurrentBrowser.contentWindow.location + "";
		    if (location.match(/^chrome:\/\/.+/)) location = "";

		    //TODO: Normalize paths for selection...
		    data = '<blockquote cite="' + location + '">' + selection_formatted.html + "</blockquote>";
		    data += '<p class="citation"><cite cite="' + location + '"><a href="' + location + '">' + title + '</a>' + "</cite></p><br/>";
		    header=location;
		    label=selection;
		    return { type: 'document', header: header, label: label, src: data, webPageUrl : webPageUrl, webPageTitle : webPageTitle };
	    }
	    else{
		    if(node.nodeName.toString().toLowerCase()=='#text'){
			    node=node.parentNode;
		    }

		    //If it is an img, make sure the src attribute is the full path
		    if(node.src){
			    node.setAttribute('src',node.src);
			    type='image';
			    label=node.src;
			    header=node.src;
			    if(node.getAttribute('alt')){
				    header=node.getAttribute('alt');
			    }
		    }
		    //If it is a link make sure the href attribute is the full path
		    if(node.href){
			    node.setAttribute('href',node.href);
			    type='link';
			    label=node.href;
			    header=node.textContent;
		    }
		    return { type: type, header: header, label: label, src: serializer.serializeToString(node) };
	    }
    }
    XXX('flock_getMarkedUpSelection: unable to understand this dropdata!!');
    return aDropData.data;

}/*}}}*/


function sendToShelf(event){/*{{{*/
    var topWindow = flock_mostRecentNavigator();
    var node = topWindow.document.popupNode;

    var contentDetector = new nsContentDetector();
    contentDetector.setTarget(node);

    var location = $$("content").mCurrentBrowser.contentWindow.location + "";
    var selection = flock.browser.getSelection();
    var title = flock.browser.currentTitle;

    var data; // Implements flockIRichObject
    var webPageUrl;

    // FIXME: send a link (not current page)
    if(contentDetector.onImage) {
        // Send selected image 
        data = new RichImage();
        webPageUrl = contentDetector.imageURL;
        // Don't insert a stupid image containing Javascript
        if (webPageUrl.match('javascript:'))
            return;
        data.init(node.alt, webPageUrl, webPageUrl);
    } else if (selection.toString().length > 0) {
        // Send selected text
        data = new RichSelection();
        data.init(selection, location, title);
    } else {
        // Default: send a link to the current page
        data = new RichLink();
        data.init(title, location);
    }
    // dump("Send to shelf "+data.src+"\n");

    flock.shelf.service.insert(data, -1);
}/*}}}*/


// This function does not really redraw, it just checks if we need
// to present the Drop Stuff Here message or the elements list
function redraw_shelf(){/*{{{*/
    var eltCount = flock.shelf.service.elementsCount();

    if(eltCount==0){
	    $('shelf-deck').setAttribute('selectedIndex', '0');
    } else {
	    $('shelf-deck').setAttribute('selectedIndex', '1');
    }
}/*}}}*/


// OBSOLETE CODE /*{{{*/ 
//function shelfPreview(){[>{{{<]
//    //dump('DRAGOVER SHELF');
//    XXX('shelf', 'preview');
//    // Show the shelf
//    $$("bottomShelfContainer").setAttribute('hidden', 'false');
//    $$('webclip-panel').previewmode = true;
//    // Check in a few seconds if we need to hide it or not
//    window.setTimeout("closeIfNeeded()", 3000);
//}[>}}}<]

//function shelfRefreshBottomLine(){[>{{{<]
//   var nbElements = flock.shelf.service.elementsCount();
//   var message;
//   switch (nbElements) {
//    case 0: message = "Web snippets: empty"; break;
//    case 1: message = "Web snippets: 1 item"; break;
//    default: message = "Web snippets:" + nbElements + " items"; break;
//   }
//   $$('webclip-panel-label').value = message;
//}[>}}}<]

//function closeIfNeeded(){[>{{{<]
//    //dump("closeIfNeeded "+$$('webclip-panel').previewmode+"\n");
//    if ($$('webclip-panel').previewmode)
//        window.setTimeout("closeIfNeeded()", 3000);
//    else
//        $$("bottomShelfContainer").setAttribute('hidden', 'true');
//}[>}}}<]

//function onShelfToggle(){[>{{{<]
//     switch ($$("bottomShelfContainer").getAttribute('hidden')){
//     case 'true': $$("bottomShelfContainer").setAttribute('hidden', 'false');
//                  $$('webclip-panel').setAttribute('open', true);
//                  break;
//     case 'false': $$("bottomShelfContainer").setAttribute('hidden', 'true');
//                  $$('webclip-panel').removeAttribute('open');
//                  break;
//     }
//}[>}}}<]

//function shelfDragExit(){[>{{{<]
//    return;
//    // dump("Exit !!\n");
//    XXX('shelf', 'exit');
//    $$('webclip-panel').previewmode = false;
//}[>}}}<]

//function shelfDragOver(){[>{{{<]
//    // dump("Entry !!\n");
//    XXX('shelf', 'over');
//    $$('webclip-panel').previewmode = true;
//}[>}}}<]
/*}}}*/
