// vim: foldmethod=marker
/*
 * BEGIN FLOCK GPL
 * 
 * Copyright Flock Inc. 2005-2006
 * http://flock.com
 * 
 * This file may be used under the terms of of the
 * GNU General Public License Version 2 or later (the "GPL"),
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 * 
 * END FLOCK GPL
 */


var gShelf;
var gShelfContext;

var gShelfCommandController = new CommandController({
    // should always work
    cmd_shelfAddNote: function () {/*{{{*/
        window.openDialog("chrome://browser/content/flock/shelf/addShelfNoteDlg.xul", "addNote", "modal");
    },/*}}}*/
    
    // ignore if there is nothing there
    cmd_shelfClear: function () {/*{{{*/
        gShelfItemCommandController.detachFrom()
        flock.shelf.service.clear();
        
        gShelf.scrollHome();
    },/*}}}*/

});


// we do not get to focus the main slidey box, so we are
// going to make the controller follow focus
var gShelfItemCommandController = new CommandController({
    // some base stuff
    cmd_copy: function () {/*{{{*/
        flock.command('cmd_shelfItemCopyHTML');
    },/*}}}*/
    cmd_delete: function () {/*{{{*/
        flock.command('cmd_shelfItemDelete');
    },/*}}}*/
    cmd_handleBackspace: function () {/*{{{*/
        flock.command('cmd_shelfItemDelete');
    },/*}}}*/
    cmd_selectAll: function () {/*{{{*/
        flock.command('cmd_shelfItemSelectAll');
    },/*}}}*/
    // XXX andy: leaving these for erwan to implement,
    //           just stubs to get to the right menu items
    //cmd_paste: function () {
    //    flock.command('cmd_shelfItemPaste');
    //},
    //cmd_cut: function () {
    //    flock.command('cmd_shelfItemCut');
    //},
    
    // XXX andy: I think these two should create proper transfer items
    //           and populate them fully
    cmd_shelfItemCopyHTML: function () {/*{{{*/
        var selected = gShelf.getSelection();
        var srcs = [];
        selected.forEach(function (aVal) {
            srcs.push(aVal.firstChild.src);
        });
        gClipboardHelper.copyString(srcs.join("\n"));
    },/*}}}*/
    cmd_shelfItemCopyLink: function () {/*{{{*/
        var selected = gShelf.getSelection();
        var urls = [];
        selected.forEach(function (aVal) {
            urls.push(aVal.firstChild.url);
        });
        gClipboardHelper.copyString(urls.join("\n"));
    },/*}}}*/
    cmd_shelfItemDelete: function () {/*{{{*/
            this.detachFrom(this._attached);
        var aSelection = gShelf.getSelection();
        
        for(var i = 0; i < aSelection.length; ++i) {
            var child = aSelection[i].firstChild;
            flock.shelf.service.remove(child.id);
        }
        if (!gShelf.canScrollRight) gShelf.scrollEnd();
    },/*}}}*/
    cmd_shelfItemSelectAll: function () {/*{{{*/
        gShelf.selectAll();
    },/*}}}*/
        
    cmd_shelfItemEdit: function () {/*{{{*/
        var theObject = gShelf.getSelection()[0].firstChild;
        window.openDialog("chrome://browser/content/flock/shelf/addShelfNoteDlg.xul", "addNote", "modal", theObject.id, theObject.content, theObject.type, theObject.url);
    },/*}}}*/

    cmd_shelfItemOpen: function () {/*{{{*/
        var item = gShelf.getSelection()[0].firstChild;
        flock.command('cmd_itemOpen', item.url);
    },/*}}}*/
    cmd_shelfItemOpenInTab: function () {/*{{{*/
        var item = gShelf.getSelection()[0].firstChild;
        flock.command('cmd_itemOpen', item.url, 'tab');
    },/*}}}*/
    cmd_shelfBlogThis: function () {/*{{{*/
        var item = gShelf.getSelection()[0].firstChild;
        var params = {
            file : 0,
            title : FLOCK_BLOG_DEFAULT_TITLE,
            content : item.src + "<p></p>"
        };
        flock_blogDoOpenPost(EDITOR_URL, params);
    },/*}}}*/
    cmd_shelfImageSendToUploader: function () {/*{{{*/
        var item = gShelf.getSelection()[0].firstChild;
        var photoUrls = new Array(); 
        photoUrls[0] = item.url;
        var dropProcessor = new PhotoDropProcessor(photoUrls, null);
        dropProcessor.start();        
    },/*}}}*/
    cmd_shelfItemOpenInWindow: function () {/*{{{*/
        var item = gShelf.getSelection()[0].firstChild;
        flock.command('cmd_itemOpen', item.url, 'window');
    },/*}}}*/
    cmd_shelfItemOpenInTabs: function () {/*{{{*/
        var selected = gShelf.getSelection();
        selected.forEach(function (aVal) {
            flock.command('cmd_itemOpen', aVal.firstChild.url, 'tab');

        });
    },/*}}}*/

    // util for our hopping behavior
    attachTo: function (aNode) {/*{{{*/
        // XXX andy: this feels mildly unreliable, I feel like I have hit
        //           most of the edge cases, but it still makes me uneasy
        if (this._attached) {
            this.detachFrom(this._attached);
            //throw "already attached";
            //return;
        }
        aNode.controllers.appendController(this);
        this.updateCommands();
        this._attached = aNode;
    },/*}}}*/
    detachFrom: function (aNode) {/*{{{*/
        if (!this._attached) {
            //throw "not attached";
            return;
        }
        if (!aNode) this._attached.controllers.removeController(this);
        aNode.controllers.removeController(this);
        this._attached = null;
    },/*}}}*/
    updateCommands: function () {/*{{{*/
        // dispatch a focus event so that the menus on top will update
        document.commandDispatcher.updateCommands('select');
        this.updateNoteNotSelected();
    },/*}}}*/
    updateNoteNotSelected: function () {/*{{{*/
        var selected = gShelf.getSelection();
        var noteNotSelected = selected.every(function (aVal) {
            if (aVal.firstChild.type == "note") return false;
            return true;
        });
        if (noteNotSelected) {
            $('bcast_shelfNoteNotSelected').setAttribute('disabled', 'false');
        }
        else {
            $('bcast_shelfNoteNotSelected').setAttribute('disabled', 'true');
        }
    },/*}}}*/

    // drag checks
    onDragDrop: function (aEvt) {/*{{{*/
        this.detachFrom(this._attached);
    },/*}}}*/
    onDragStart: function (aEvt) {/*{{{*/
        this.detachFrom(this._attached);
    },/*}}}*/

});


function onLoad_shelf(aEvt) {/*{{{*/
    XXX('onLoad_shelf');

    // Main controller
    // gShelf = new flock.xul.ShelfController($('shelfContainer'), {
    gShelf = new flock.shelf.controller($('shelfContainer'), $('webclip-panel'),
            $('flock-createSnippet-button'), $('flock-clearSnippets-button'),
            $('webclip-deck'), $('icon-view'));

    gShelf.refreshPanel();
    
    // RDF Updater
    window.gShelfRDFObserver = new flock.rdf.BatchedRDFObserver({
        onAssertBatch: function () { gShelf.refreshPanel(); },
        onUnassertBatch: function () { gShelf.refreshPanel(); },
        onChangeBatch: function () { gShelf.refreshPanel(); }, 
    });
    flock.shelf.service.AddObserver(window.gShelfRDFObserver);
    
    // Commands   
    window.controllers.appendController(gShelfCommandController);
    
    // Context Menu
    gShelfContext = new flock.xul.SwitchingPopup($('shelfContext'), {
        isDebug: true,
        popupSet: "shelfContextPopupSet",
        onPopupShowing: function (aEvt) {/*{{{*/
            var count = gShelf.getSelection().length;
            if (count == 0) {
                this.switchTo("shelfEmptyContext");
            }
            else if (count == 1) {
                var shelfIcon = gShelf.getSelection()[0].firstChild;
                switch (shelfIcon.type) {
                  case "image": this.switchTo("shelfImageContext");break;
                  case "document": this.switchTo("shelfDocumentContext");break;
                  case "link": this.switchTo("shelfLinkContext");break;
                  case "note": this.switchTo("shelfNoteContext");break;
                  default: this.switchTo("shelfSingleContext");
                }
            }
            else {
                this.switchTo("shelfMultipleContext");
            }
            return true;
        },/*}}}*/
    });
    //gShelfContext.attach();
    

    // XXX andy: hack to get around faulty slideybox keystroke handling
    $('webclip-deck').addEventListener('keydown', gShelf.preventSlideyboxKeystrokesHack, true);
}/*}}}*/
function onUnload_shelf(aEvt) {/*{{{*/
    XXX('onUnload_shelf');
    flock.shelf.service.RemoveObserver(window.gShelfRDFObserver);
}/*}}}*/

function onShelfItemDlbclick(shelfIcon){
    switch (shelfIcon.type) {
      case "image": goDoCommand('cmd_shelfItemOpen');break;
      default: goDoCommand('cmd_shelfItemEdit');
    }
}

window.addEventListener('load', onLoad_shelf, false);
window.addEventListener('unload', onUnload_shelf, false);

